# -*- coding: utf-8 -*-

"""
REF
https://qiita.com/c60evaporator/items/e1ecccab07a607487dcf#テスト用pypiへのライブラリアップロード
https://zenn.dev/sikkim/articles/490f4043230b5a
https://hack.nikkei.com/blog/advent20211225/#Cythonを含んだmoduleのinstall例

$ pip install pipreqs
$ pipreqs .
-> manually pasted into INSTALL_REQUIREMENTS with some modification
$ pip install twine
$ pip install wheel
$ python setup.py sdist
-> `dist`, `savemoney.egg-info` directories will be created.
$ python setup.py bdist_wheel
-> `build` directory will be created.
$ twine upload --repository testpypi dist/*

To install parasail, 
$ brew install libtool
$ brew install automake
might be required beforehand.

"""

import re
from pathlib import Path
from setuptools import setup, find_packages, Extension

try:
    import numpy as np
except ImportError:
    from setuptools import dist
    dist.Distribution().fetch_build_eggs(['numpy>=1.26.2'])
    import numpy as np

try:
    from Cython.Build import cythonize
except ImportError:
    from setuptools import dist
    dist.Distribution().fetch_build_eggs(['Cython>=3.0.7'])
    from Cython.Build import cythonize

#############################################################
#############################################################

test_version = ""
root_dir = Path(__file__).parent.resolve()
package_name = "savemoney"
sub_package_names = [f"{package_name}.{sub_package_name}" for sub_package_name in find_packages(package_name)]

class About():
    def __init__(self, about_path, keys) -> None:
        self.keys = keys
        with open(about_path) as f:
            text = f.read()
        for key in keys:
            m = re.search(fr'__{key}__\s*=\s*[\'\"](.+?)[\'\"]', text)
            setattr(self, key, m.group(1))
    def __str__(self) -> str:
        txt = ""
        for key in self.keys:
            txt += getattr(self, key) + "\n"
        return txt.strip()
    @property
    def version_pypi(self):
        return re.match(r"ver_([0-9.]+)", self.version).group(1) + test_version   # version format must be "x.x.x" etc.

about = About(
    root_dir / package_name / '__about__.py', 
    [
        "copyright", 
        "version", 
        "license", 
        "author", 
        "email", 
        "url", 
        "appname", 
        "comment", 
        "description", 
    ]
)

with open('README.md', encoding='utf-8') as f:
    long_description = f.read()

INSTALL_REQUIREMENTS = [
    "pandas>=1.5.3", 
    "parasail>=1.3.4", 
    "Pillow>=9.4.0", 
    "PuLP>=2.7.0", 
    "scipy>=1.11.4", 
    # "setuptools>=68.2.2", 
    "snapgene_reader>=0.1.20", 
    "tqdm>=4.66.1", 
    # "Bio>=1.6.0",         # ?
    "biopython>=1.83", 
    "Cython>=3.0.7", 
    "matplotlib>=3.7.1", 
    "numpy>=1.23.5", 
    # "oauth2client>=4.1.3", 
    # "pysam>=0.22.0", 
    "pyspoa>=0.2.1"
]

PYTHON_REQUIRES = ">=3.10"

setup(
    name = package_name, 
    packages = [package_name] + sub_package_names, 
    ###
    copyright = about.copyright, 
    version = about.version_pypi, 
    license = about.license, 
    author = about.author, 
    author_email = about.email, 
    maintainer = about.author, 
    maintainer_email = about.email, 
    url = about.url, 
    description = about.description, 
    ###
    long_description=long_description, 
    long_description_content_type='text/markdown',
    keywords = (
        "plasmid, "
        "whole-plasmid, "
        "sequencing, "
        "alignment, "
        "multiplexing, "
        "barcode-free, "
        "bayesian analysis, "
        "prior information, "
    ), 

    install_requires = INSTALL_REQUIREMENTS, 
    python_requires = PYTHON_REQUIRES, 
    
    classifiers = [
        'Development Status :: 4 - Beta', 
        'License :: Free for non-commercial use', 
        "Programming Language :: Cython", 
        'Programming Language :: Python', 
        'Programming Language :: Python :: 3', 
        'Programming Language :: Python :: 3.10', 
        'Topic :: Scientific/Engineering :: Bio-Informatics', 
    ],
    ext_modules = cythonize([Extension("savemoney.modules.cython_functions.alignment_functions", ["savemoney/modules/cython_functions/alignment_functions.pyx"])], compiler_directives = {"language_level": 3, "embedsignature": True}), 
    include_dirs = np.get_include(),    # cython に numpy の path を通す
    include_package_data = True         # 配布時にpyxのファイルを含める (MANIFEST.in で定義)
)






# -*- coding: utf-8 -*-

from typing import List
from pathlib import Path
from itertools import chain

# my modules
from ..modules import my_classes as mc
from . import post_analysis_core as pac

__all__ = ["post_analysis", "default_post_analysis_param_dict", "post_analysis_separate_paths_input"]

error_rate = 0.0000001
default_post_analysis_param_dict = {
    'gap_open_penalty': 3, 
    'gap_extend_penalty': 1, 
    'match_score': 1, 
    'mismatch_score': -2, 
    'score_threshold': 0.3, 
    'error_rate': error_rate, 
    'del_mut_rate': error_rate / 4, # e.g. "A -> T, C, G, del"
    'ins_rate': error_rate, 
    'window': 160,      # maximum detectable length of repetitive sequences: if region of 80 nt is repeated adjascently two times, put the value of 160.
}

def post_analysis(sequence_dir_path:str, save_dir_base: str, **param_dict:dict):
    plasmid_map_paths = [path for path in Path(sequence_dir_path).glob(f"*.*") if path.suffix in mc.MyRefSeq.allowed_plasmid_map_extensions]
    fastq_paths = list(Path(sequence_dir_path).glob(f"*.fastq"))
    return post_analysis_separate_paths_input(plasmid_map_paths, fastq_paths, save_dir_base, **param_dict)

def post_analysis_separate_paths_input(plasmid_map_paths:List[Path], fastq_paths:List[Path], save_dir_base: str, **param_dict:dict):
    for k in param_dict.keys():
        if k not in default_post_analysis_param_dict.keys():
            raise Exception(f"unknown key in `param_dict`: {k}\nallowed keys are: {', '.join(default_post_analysis_param_dict.keys())}")
    param_dict = {key: param_dict.get(key, val) for key, val in default_post_analysis_param_dict.items()}
    mc.assert_param_dict(param_dict)
    if len(plasmid_map_paths) == 0:
        raise Exception(f"Error: No plasmid map file was detected!")
    if len(fastq_paths) == 0:
        raise Exception(f"Error: No fastq file was detected!")
    # 1. Prepare objects
    ref_seq_list = [mc.MyRefSeq(plasmid_map_path) for plasmid_map_path in plasmid_map_paths]
    my_fastq = mc.MyFastQ.combine([mc.MyFastQ(fastq_path) for fastq_path in fastq_paths])
    save_dir = mc.new_dir_path_wo_overlap(Path(save_dir_base) / my_fastq.combined_name_stem, spacing="_")
    save_dir.mkdir()
    # 2. Execute alignment: load if any previous score_matrix if possible
    result_dict, ref_seq_list = pac.execute_alignment(ref_seq_list, my_fastq, param_dict, save_dir)
    # 3. normalize alignment score and set threshold for assignment
    query_assignment = pac.normalize_scores_and_apply_threshold(ref_seq_list, my_fastq, result_dict, param_dict)
    pac.draw_and_save_query_assignment(query_assignment, save_dir, display_plot=False)
    # 4. MSA/consensus
    my_msa_list = pac.execute_msa(result_dict, query_assignment, param_dict)
    # 5. EXPORT
    pac.export_results(my_msa_list, save_dir)
    pac.export_log(ref_seq_list, my_fastq, param_dict, query_assignment, save_dir)
    return save_dir



# -*- coding: utf-8 -*-

from savemoney.__about__ import (
    __author__, 
    __copyright__, 
    __email__, 
    __license__, 
    __appname__, 
    __url__, 
    __version__, 
    __comment__, 
    __description__, 
)

__all__ = [
    "__author__", 
    "__copyright__", 
    "__email__", 
    "__license__", 
    "__appname__", 
    "__url__", 
    "__version__", 
    "__description__",  
    "__comment__", 
]

from .pre_survey import *
from .post_analysis import *
from .show_consensus import *
from .ca2bam import *



# -*- coding: utf-8 -*-

"""
$python test/test_local_all.py
"""


import sys
from pathlib import Path
abs_path = Path(__file__).resolve()
sys.path.insert(0, abs_path.parents[1].as_posix())
import subprocess

import savemoney

if __name__ == "__main__":
    sequence_dir_path = abs_path.parents[1] / "resources/demo_data_1_subset/demo_data_1_subset_input/my_plasmid_maps_fa"
    # sequence_dir_path = abs_path.parents[1] / "resources/demo_data/my_plasmid_maps_dna"
    save_dir_base = abs_path.parents[1] / "resources/demo_data"

    consensus_alignment_path = abs_path.parents[1] / "resources/demo_data/Uematsu_n7x_1_MU-test1_subset/M32_pmNeonGreen-N1.fa.ca"

    ###############
    # WITH IMPORT #
    ###############
    param_dict = {  # optional params
        'distance_threshold':   5, 
        'number_of_groups':     1, 
    }
    savemoney.pre_survey(sequence_dir_path, save_dir_base, **param_dict)
    savemoney.post_analysis(sequence_dir_path, save_dir_base)
    savemoney.show_consensus(consensus_alignment_path)
    savemoney.ca2bam(consensus_alignment_path)

    #####################
    # WITH COMMAND LINE #
    #####################
    subprocess.call(f"python -m savemoney.pre_survey {sequence_dir_path} {save_dir_base} -dt {param_dict['distance_threshold']} -nog {param_dict['number_of_groups']}", shell=True)
    subprocess.call(f"python -m savemoney.post_analysis {sequence_dir_path} {save_dir_base}", shell=True)
    subprocess.call(f"python -m savemoney.show_consensus {consensus_alignment_path}", shell=True)
    subprocess.call(f"python -m savemoney.ca2bam {consensus_alignment_path}", shell=True)




